@@ -40,7 +40,11 @@ module Bullet
                 :stacktrace_excludes,
                 :skip_html_injection
     attr_reader :safelist
-    attr_accessor :add_footer, :orm_patches_applied, :skip_http_headers, :always_append_html_body
+    attr_accessor :add_footer,
+                  :orm_patches_applied,
+                  :skip_http_headers,
+                  :always_append_html_body,
+                  :skip_user_in_notification
 
     available_notifiers =
       UniformNotifier::AVAILABLE_NOTIFIERS.select { |notifier| notifier != :raise }
