@@ -33,6 +33,8 @@ module Bullet
             'active_record70'
           elsif active_record71?
             'active_record71'
+          elsif active_record72?
+            'active_record72'
           else
             raise "Bullet does not support active_record #{::ActiveRecord::VERSION::STRING} yet"
           end
@@ -114,6 +116,10 @@ module Bullet
       active_record7? && ::ActiveRecord::VERSION::MINOR == 1
     end
 
+    def active_record72?
+      active_record7? && ::ActiveRecord::VERSION::MINOR == 2
+    end
+
     def mongoid4x?
       mongoid? && ::Mongoid::VERSION =~ /\A4/
     end
