@@ -2,6 +2,7 @@
 
 module Bullet
   module Detector
-    class Base; end
+    class Base
+    end
   end
 end
