@@ -1,3 +1,4 @@
 # frozen_string_literal: true
 
-class Folder < Document; end
+class Folder < Document
+end
